package zero1hd.polyjet.ui.pages;

import java.io.File;
import java.io.FilenameFilter;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.polyjet.Polyjet;
import zero1hd.polyjet.screens.MainMenu;
import zero1hd.polyjet.ui.builders.MusicSelectable;

public class MusicSelectionPage extends Page {
	private volatile Image loading;
	private volatile Table musicChoices;
	Preferences musicFileAnnotation;
	private Polyjet core;
	private volatile Label status;
	private volatile ScrollPane musicChoiceScroller;
	
	public MusicSelectionPage(final Polyjet core, final MainMenu mainMenu) {
		super("Select music", core.defaultSkin);
		this.core = core;
		
		musicFileAnnotation = Gdx.app.getPreferences("music_file_annotation");
		
		
		TextButton back = new TextButton("Back", core.defaultSkin);
		back.setPosition(getWidth()-back.getWidth()-15f, getHeight()-back.getHeight()-15f);
		back.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				core.setScreen(mainMenu);
			}
		});
		addActor(back);
		
		loading = new Image(core.defaultSkin, "loading");
		loading.setPosition((getWidth()-loading.getWidth())/2, getHeight()-loading.getHeight() - 32f);
		loading.setOrigin(loading.getWidth()/2, loading.getHeight()/2);
		loading.addAction(Actions.forever(Actions.rotateBy(-360f, 2f)));
		addActor(loading);
		
		status = new Label("Loading: ", core.defaultSkin, "sub-font", core.defaultSkin.getColor("default"));
		status.setPosition(loading.getX()+loading.getWidth()+12f, loading.getY());
		addActor(status);
		
		
		musicChoices = new Table();
		musicChoices.defaults().pad(10f);
		musicChoiceScroller = new ScrollPane(musicChoices);
		musicChoiceScroller.setScrollingDisabled(false, true);
		musicChoiceScroller.setSize(getWidth(), getHeight()*0.85f);
		addActor(musicChoiceScroller);
	}
	
	@Override
	public void act(float delta) {
		
		super.act(delta);
	}
	
	public void beginMusicSearch() {
		new Thread(new Runnable() {
			
			@Override
			public void run() {
				FileHandle[] musicFiles = new FileHandle(core.prefs.getString("music dir")).list(new FilenameFilter() {
					@Override
					public boolean accept(File dir, String name) {
						if (name.endsWith("mp3") || name.endsWith("wav")) {
							return true;
						}
						return false;
					}
				});
				
				if (musicFiles.length != 0) {
					for (int music = 0; music < musicFiles.length; music++) {
						final MusicSelectable selectable = new MusicSelectable(musicFiles[music], musicFileAnnotation, core.defaultSkin, core.assetManager.get("defaultCover.png", Texture.class));
						Gdx.app.postRunnable(new Runnable() {
							@Override
							public void run() {
								Gdx.app.debug("Music Search Thread", "Finished loading: " + selectable.getName());
								musicChoices.add(selectable).prefSize(256, 0.8f*getHeight());
								selectable.addInfoToPanel(musicChoiceScroller);
							}
						});
						System.out.println(music);
						int prog = (int) (100f*music/(musicFiles.length-1f));
						status.setText("Loading: " + prog + "%");
					}
					
					status.remove();
					loading.remove();
					
				} else {
					loading.clearActions();
					status.setText("No music found.");
				}
			}
		}).start();
		
		
	}
}
